/*
* lire un unique capteur DS18B20 sur un bus 1-Wire.
 */

int i = 0;

/* Dépendance pour le bus 1-Wire */
#include <OneWire.h>
 
/* Broche du bus 1-Wire */
const byte BROCHE_ONEWIRE = A1;
/* Adresses des capteurs de température */
const byte SENSOR_ADDRESS_1[] = { 0x28, 0xAA, 0x66, 0x99, 0x3B, 0x14, 0x01, 0xE4 };
const byte SENSOR_ADDRESS_2[] = { 0x28, 0xAA, 0x9D, 0x91, 0x3B, 0x14, 0x01, 0x2E };
const byte SENSOR_ADDRESS_3[] = { 0x28, 0xAA, 0xA7, 0x80, 0x3B, 0x14, 0x01, 0xA0 };


/* Code de retour de la fonction getTemperature() */
enum DS18B20_RCODES {
  READ_OK,  // Lecture ok
  NO_SENSOR_FOUND,  // Pas de capteur
  INVALID_ADDRESS,  // Adresse reçue invalide
  INVALID_SENSOR  // Capteur invalide (pas un DS18B20)
};


/* Création de l'objet OneWire pour manipuler le bus 1-Wire */
 OneWire ds(BROCHE_ONEWIRE);


byte getTemperature(float* temperature, byte reset_search, byte addr[]){
   byte data[9];

   /* Reset le bus 1-Wire ci nécessaire (requis pour la lecture du premier capteur) */
  if (reset_search) {
    ds.reset_search();
  }
   /* Reset le bus 1-Wire et sélectionne le capteur */
  ds.reset();
  ds.select(addr);

   /* Lance une prise de mesure de température et attend la fin de la mesure */
  ds.write(0x44, 1);
  delay(800);
  
  /* Reset le bus 1-Wire, sélectionne le capteur et envoie une demande de lecture du scratchpad */
  ds.reset();
  ds.select(addr);
  ds.write(0xBE);
 
 /* Lecture du scratchpad */
  for (byte i = 0; i < 9; i++) {
    data[i] = ds.read();
  }
   
  /* Calcul de la température en degré Celsius */
  *temperature = ((data[1] << 8) | data[0]) * 0.0625; 
  
   
  // Pas d'erreur
  return READ_OK;
}
/**
 * Fonction de lecture de la température via un capteur DS18B20.
 */
byte getTemperatures(float *temperature1, float* temperature2, float* temperature3, byte reset_search){
    getTemperature(temperature1, reset_search, SENSOR_ADDRESS_1);
    getTemperature(temperature2, reset_search, SENSOR_ADDRESS_2);
    getTemperature(temperature3, reset_search, SENSOR_ADDRESS_3);
    return READ_OK;
}

/** Fonction setup() **/
void setup() {

  /* Initialisation du port série */
  Serial.begin(9600);

    //Serial.println("CLEARDATA"); // clears sheet starting at row 2
    Serial.println("CLEARSHEET"); // clears sheet starting at row 1
    
  // define 5 columns named "Date", "Time", "Timer", "Counter" and "millis"
    Serial.println("LABEL,Date,Time,Timer,Temperature1,Temperature2,Temperature3");

  // set the names for the 3 checkboxes
    Serial.println("CUSTOMBOX1,LABEL,Stop logging at 250?");
    Serial.println("CUSTOMBOX2,LABEL,Resume log at 350?");
    Serial.println("CUSTOMBOX3,LABEL,Quit at 450?");

  // check 2 of the 3 checkboxes (first two to true, third to false)
    Serial.println("CUSTOMBOX1,SET,0");
    Serial.println("CUSTOMBOX2,SET,0");
    Serial.println("CUSTOMBOX3,SET,0");
}

/** Fonction loop() **/
void loop() {
  float temperature1;
  float temperature2;
  float temperature3;

  /* Lit la température ambiante à ~1Hz */
  if (getTemperatures(&temperature1,&temperature2,&temperature3, true) != READ_OK) {
    Serial.println(F("Erreur de lecture du capteur"));
    return;
  }

  /* Affiche la température */
  //Serial.print(F("Temperature : "));
  //Serial.print(temperature, 2);
  //Serial.write(176); // Caractère degré
  //Serial.write('C');
  //Serial.println();


// Mesure la tension sur la broche D2?
 // int temperature = 2;
  
    // simple print out of number and millis. Output e.g.,: "DATA,DATE,TIME,TIMER,4711,13374,AUTOSCROLL_20"
      //Serial.println( (String) "DATA,DATE,TIME,TIMER," + i++ + "," + millis() + ",AUTOSCROLL_20" );
     // Serial.println( (String) "DATA,DATE,TIME,TIMER," + temperature_celcius + "," + millis() + ",AUTOSCROLL_20" );
        Serial.println( (String) "DATA,DATE,TIME,TIMER," + temperature1 +","+ temperature2 + "," + temperature3 + "," + "AUTOSCROLL_20");
      // alternative writing method:
        /* Serial.print("DATA,DATE,TIME,TIMER,");
        Serial.print(i++); Serial.print(",");
        Serial.println(millis()); 
        Serial.print(","); Serial.println("SCROLLDATA_20"); */
    delay(200);
    
    // clear some cells in Excel (rectangle range from B10 to D20)
      if(i==100)
        Serial.println("ClearRange,B,10,D,20");

    // do a simple beep in Excel on PC
      if(i==150)
        Serial.println("BEEP");

    // read a value (in this case integer) from Excel (from a sheet by name)
      if(i==200)
      {
        Serial.println("CELL,GET,FROMSHEET,Simple Data,E,4"); // ==> request value from sheet
          // Serial.println("CELL,GET,E4"); ==> short version to read from active sheet in Excel
        int readvalue = Serial.readStringUntil(10).toInt(); // get response. Note: the '10' is important! Always use but never change ;-)
        Serial.println( (String) "Value of cell E4 is: " + readvalue); // result displayed in Excel DirectDebugWindow to double check
      }

    // check value of custombox1 on PLX DAQ in Excel and if
    // checkbox is checked then send the command to pause logging
      if(i==250)
      {
        Serial.println("CUSTOMBOX1,GET");
        int stoplogging = Serial.readStringUntil(10).toInt();
        // this information can be seen in the direct debug window on PLX DAQ in Excel
        Serial.println( (String) "Value of stoplogging/checkbox is: " + stoplogging);
        if(stoplogging) 
          Serial.println("PAUSELOGGING");
      }

    // get a true random number from the computer
      if(i==300)
      {
        Serial.println("GETRANDOM,-4321,12345"); // between -4321 to 12345
        int rndseed = Serial.readStringUntil(10).toInt();
        Serial.println( (String) "Got random value '" + rndseed + "' from Excel" );
        // Note: this information is not posted to the Excel sheet because "DATA" is missing
        // instead this information can be seen in the direct debug window
      }

    // and now resume logging
      if(i==350)
      {
        Serial.println("CUSTOMBOX2,GET");
        int resumelogging = Serial.readStringUntil(10).toInt();
        if(resumelogging) 
          Serial.println("RESUMELOGGING");
      }   

    // post to specific cells on default sheet as well as named sheet
      if(i==400)
      {
        Serial.println("CELL,SET,G10,400 test 1 string"); // default sheet active in PLX DAQ Excel
        Serial.println("CELL,SET,ONSHEET,Simple Data,G,11,400 test 2 string"); // named sheet available in PLX DAQ Excel
      }
       
    // and for forced quit of Excel with saving the file first
      if(i==450)
      {
        Serial.println("CUSTOMBOX3,GET");
        if(Serial.readStringUntil(10).toInt()) {
          Serial.println("SAVEWORKBOOKAS,450-Lines-File");
          Serial.println("FORCEEXCELQUIT");
        }
        else
          Serial.println("No forced Excel quit requested!");
      }

}
